"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const Definition_1 = require("../metadata/Definition");
const InputField_1 = require("../reference/InputField");
const types_1 = require("../types");
/**
 * Metadata type for InputObjectType
 */
class InputType extends Definition_1.Definition {
    constructor() {
        super(...arguments);
        this.instantiationCache = new WeakMap();
    }
    get kind() { return 'input'; }
    buildTypeInstance(context) {
        const name = this.definitionName;
        const description = this.description;
        const fields = this.buildInputFieldConfigMap.bind(this, context);
        return new graphql_1.GraphQLInputObjectType({ name, fields, description });
    }
    buildInputFieldConfigMap(context) {
        const fields = this.findReference(context, InputField_1.InputField);
        return fields.reduce((results, field) => {
            results[field.fieldName] = this.buildInputFieldConfig(context, field);
            return results;
        }, {});
    }
    buildInputFieldConfig(context, field) {
        return {
            type: field.resolveType(context.storage),
            defaultValue: field.defaultValue,
            description: field.description,
        };
    }
    buildInstantiator(context) {
        if (!this.definitionClass) {
            return types_1.defaultInputFieldInstantiator;
        }
        const fields = this.findReference(context, InputField_1.InputField);
        const fieldInstantiators = fields.reduce((res, field) => {
            res[field.fieldName] = field.buildInstantiator(context.storage);
            return res;
        }, {});
        const instantiator = (values) => {
            let cached = this.instantiationCache.get(values);
            if (!cached) {
                cached = new this.definitionClass();
                Object.keys(values).forEach(fieldName => {
                    cached[fieldName] = fieldInstantiators[fieldName](values[fieldName]);
                });
                this.instantiationCache.set(values, cached);
            }
            return this.instantiationCache.get(values);
        };
        Object.defineProperty(instantiator, 'name', { value: 'instantiate' + this.definitionName });
        return instantiator;
    }
}
exports.InputType = InputType;
//# sourceMappingURL=InputType.js.map