"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const Definition_1 = require("../metadata/Definition");
const Field_1 = require("../reference/Field");
/**
 * Metadata type for InterfaceType
 */
class InterfaceType extends Definition_1.Definition {
    get kind() { return 'output'; }
    buildFieldConfig(context, field) {
        const { description, deprecationReason } = field;
        return {
            type: field.resolveType(context.storage),
            args: field.buildArgumentMap(context),
            resolve: field.buildResolver(context),
            description,
            deprecationReason,
        };
    }
    buildFieldConfigMap(context) {
        const fields = this.findReference(context, Field_1.Field);
        return (fields.reduce((results, field) => {
            const name = field.fieldName;
            results[name] = this.buildFieldConfig(context, field);
            return results;
        }, {}));
    }
    buildTypeInstance(context) {
        const name = this.definitionName;
        const description = this.description;
        const fields = this.buildFieldConfigMap.bind(this, context);
        return new graphql_1.GraphQLInterfaceType({ name, fields, description });
    }
}
exports.InterfaceType = InterfaceType;
//# sourceMappingURL=InterfaceType.js.map