"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const Definition_1 = require("../metadata/Definition");
const Field_1 = require("../reference/Field");
const Implement_1 = require("../reference/Implement");
/**
 * Metadata type for ObjectType
 */
class ObjectType extends Definition_1.Definition {
    get kind() { return 'output'; }
    /**
     * Build GraphQLObjectType instance from metadata.
     */
    buildTypeInstance(context) {
        const name = this.definitionName;
        const description = this.description;
        const fields = this.buildFieldConfigMap.bind(this, context);
        const interfaces = this.findInterfaces(context);
        const isTypeOf = this.buildIsTypeOf(context);
        return new graphql_1.GraphQLObjectType({ name, fields, interfaces, description, isTypeOf });
    }
    buildFieldConfigMap(context) {
        const fields = this.findReference(context, Field_1.Field);
        return (fields.reduce((results, field) => {
            const name = field.fieldName;
            results[name] = this.buildFieldConfig(context, field);
            return results;
        }, {}));
    }
    buildFieldConfig(context, field) {
        const { description, deprecationReason } = field;
        return {
            type: field.resolveType(context.storage),
            args: field.buildArgumentMap(context),
            resolve: field.buildResolver(context),
            description,
            deprecationReason,
        };
    }
    findInterfaces(context) {
        const impls = this.findReference(context, Implement_1.Implement);
        return impls.map(impl => impl.resolveType(context.storage));
    }
    buildIsTypeOf(context) {
        const { definitionClass } = this;
        if (definitionClass instanceof Function) {
            return (source) => (source instanceof definitionClass);
        }
        else {
            return () => true;
        }
    }
}
exports.ObjectType = ObjectType;
//# sourceMappingURL=ObjectType.js.map