"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rootTypes_1 = require("./definition/rootTypes");
const scalar_1 = require("./definition/scalar");
const MetadataStorage_1 = require("./metadata/MetadataStorage");
const coerceType_1 = require("./type-expression/coerceType");
/**
 * Create a new MetadataStorage initialized with default metadata
 */
function createMetadataStorage() {
    const storage = new MetadataStorage_1.MetadataStorage();
    scalar_1.loadBuiltInScalar(storage);
    rootTypes_1.loadFallbackRootTypes(storage);
    return storage;
}
exports.createMetadataStorage = createMetadataStorage;
function getGlobalMetadataStorage() {
    if (!exports.globalMetadataStorage) {
        exports.globalMetadataStorage = createMetadataStorage();
    }
    return exports.globalMetadataStorage;
}
exports.getGlobalMetadataStorage = getGlobalMetadataStorage;
/**
 * Get a GraphQLType instance from the given storage or default
 * global metadata storage.
 * @param typeArg
 * @param storage
 */
function getType(typeArg, kind = 'any') {
    const storage = getGlobalMetadataStorage();
    return coerceType_1.coerceType(typeArg).getType({ kind, storage });
}
exports.getType = getType;
/**
 * Define a type linked to decorated class and add it to the given
 * storage or default global metadata storage.
 * @param metadataOrThunk
 */
function defineType(parsersOrThunk) {
    const storage = getGlobalMetadataStorage();
    return function defDecoratorFn(definitionClass) {
        storage.deferRegister(() => {
            (Array.isArray(parsersOrThunk) ? parsersOrThunk : parsersOrThunk()).forEach(parser => {
                storage.registerMetadata(parser.parseWithResolverMap(definitionClass));
            });
        });
        return definitionClass;
    };
}
exports.defineType = defineType;
//# sourceMappingURL=global.js.map