"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
/**
 * Contain configs required to build named GraphQL types.
 * Guarantee its type instance only created once.
 */
class Definition {
    constructor(config) {
        this.config = config;
    }
    get kind() { return 'any'; }
    get definitionClass() { return this.config.definitionClass; }
    get definitionName() { return this.config.definitionName; }
    get description() { return this.config.description; }
    getOrCreateTypeInstance(context) {
        if (!this.graphqlType) {
            this.graphqlType = this.buildTypeInstance(context);
        }
        return this.graphqlType;
    }
    findReference(context, referenceClass) {
        const referenceByName = context.storage.findReference(referenceClass, this.definitionName);
        const referenceByClass = this.definitionClass
            ? context.storage.findReference(referenceClass, this.definitionClass)
            : [];
        return [...referenceByName, ...referenceByClass];
    }
    /**
     * Build GraphQLType instance from metadata.
     */
    buildTypeInstance(context) {
        throw new Error(`Should implement typeInstance getter in ${this.constructor.name}`);
    }
    buildInstantiator(context) {
        return types_1.defaultInputFieldInstantiator;
    }
}
exports.Definition = Definition;
//# sourceMappingURL=Definition.js.map