"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Definition_1 = require("./Definition");
const Reference_1 = require("./Reference");
/**
 * Keep all [[Definition]] and references.
 * Provide methods to query metadata with its associated class or graphql type name.
 */
class MetadataStorage {
    constructor() {
        this.deferredRegister = [];
        this.deferredResolved = false;
        this.definitions = [];
        this.references = [];
    }
    deferRegister(metadataFn) {
        this.deferredRegister.unshift(metadataFn);
    }
    resolveDeferred() {
        while (this.deferredRegister.length > 0) {
            this.deferredRegister.pop()();
        }
    }
    registerMetadata(metadata) {
        metadata.map(entry => {
            if (entry instanceof Definition_1.Definition) {
                this.definitions.push(entry);
            }
            else if (entry instanceof Reference_1.Reference) {
                this.references.push(entry);
            }
        });
    }
    /**
     *
     * Get a [[Definition]] object which is instance of the `metadataClass` and associated to `linkedClass`
     * @param metadataClass A [[Definition]] subclass to query
     * @param definitionKey A class associated with metadata to query
     * @param asKind
     */
    getDefinition(metadataClass, definitionKey, asKind) {
        this.resolveDeferred();
        const entry = this.definitions.find(metadata => {
            let classOrNameMatched;
            if (typeof definitionKey === 'string') {
                classOrNameMatched = metadata.definitionName === definitionKey;
            }
            else {
                classOrNameMatched = metadata.definitionClass === definitionKey;
            }
            const typeMatched = asKind === 'any' || metadata.kind === 'any' || metadata.kind === asKind;
            return classOrNameMatched && (metadata instanceof metadataClass) && typeMatched;
        });
        return entry;
    }
    findReference(metadataClass, definitionKey) {
        this.resolveDeferred();
        const entries = this.references.filter(metadata => {
            let classOrNameMatched;
            if (definitionKey instanceof Function) {
                classOrNameMatched = metadata.source instanceof Function && equalsOrInherits(definitionKey, metadata.source);
            }
            else {
                classOrNameMatched = metadata.source === definitionKey;
            }
            // const typeMatched = metadata.kind === asKind;
            return classOrNameMatched && (metadata instanceof metadataClass);
        });
        return entries;
    }
}
exports.MetadataStorage = MetadataStorage;
function equalsOrInherits(cls, superClass) {
    return cls === superClass || cls.prototype instanceof superClass;
}
exports.equalsOrInherits = equalsOrInherits;
//# sourceMappingURL=MetadataStorage.js.map