"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Reference_1 = require("../metadata/Reference");
class Field extends Reference_1.Reference {
    get kind() { return 'output'; }
    get fieldName() { return this.config.fieldName; }
    get description() { return this.config.description; }
    get deprecationReason() { return this.config.deprecationReason; }
    get resolver() { return this.config.resolver; }
    buildArgumentMap(context) {
        const { args } = this.config;
        return args.reduce((args, ref) => {
            args[ref.fieldName] = {
                type: ref.resolveType(context.storage),
                defaultValue: ref.defaultValue,
                description: ref.description,
            };
            return args;
        }, {});
    }
    buildResolver(context) {
        const { args, resolver } = this.config;
        const instantiators = args.reduce((res, meta) => {
            res[meta.fieldName] = meta.buildInstantiator(context.storage);
            return res;
        }, {});
        const argumentInstantiator = (argValues) => {
            return Object.keys(argValues).reduce((res, fieldName) => {
                res[fieldName] = instantiators[fieldName](argValues[fieldName]);
                return res;
            }, {});
        };
        const finalResolver = function (source, args, context, info) {
            return resolver(source, argumentInstantiator(args), context, info);
        };
        return finalResolver;
    }
}
exports.Field = Field;
//# sourceMappingURL=Field.js.map