"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const TypeExpression_1 = require("./TypeExpression");
const Definition_1 = require("../metadata/Definition");
/**
 * Contain an argument which can be resolved to GraphQLType instance.
 */
class DefinitionTypeExpression extends TypeExpression_1.TypeExpression {
    constructor(typeArg) {
        super();
        this.typeArg = typeArg;
    }
    getTypeName(context) {
        return this.resolveDefinition(context).definitionName;
    }
    getType(context) {
        return this.resolveDefinition(context).getOrCreateTypeInstance(context);
    }
    getInstantiator(context) {
        let def;
        try {
            def = this.resolveDefinition(context);
        }
        catch (_a) { }
        return def ? def.buildInstantiator(context) : types_1.defaultInputFieldInstantiator;
    }
    resolveDefinition(context) {
        const def = context.storage.getDefinition(Definition_1.Definition, this.typeArg, context.kind);
        if (!def) {
            throw new Error(`Cannot resolve type: ${this.typeArg}`);
        }
        return def;
    }
}
exports.DefinitionTypeExpression = DefinitionTypeExpression;
//# sourceMappingURL=DefinitionTypeExpression.js.map