"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Class-based resolver context
 */
class ResolverContext {
    constructor(_base, _context, _info) {
        this.$env = arguments;
    }
    get $source() {
        return this.$env[0];
    }
    get $context() {
        return this.$env[1];
    }
    get $info() {
        return this.$env[2];
    }
}
exports.ResolverContext = ResolverContext;
/**
 * Decorator for fields resolved from source directly
 * @param fieldName
 */
function source(fieldName) {
    return function (prototype, propertyKey) {
        const get = function () {
            return this.$source[fieldName || propertyKey];
        };
        const set = function (value) {
            this.$source[fieldName || propertyKey] = value;
        };
        Object.defineProperty(prototype, propertyKey, { get, set });
    };
}
exports.source = source;
/**
 * Decorator for field resolved after async fetching
 * prototype should have `$fetch()` method which returns a Promise
 * @param fieldName
 */
function lazy(fieldName) {
    return function (prototype, propertyKey) {
        const get = function () {
            if (!this.$__fetcher__) {
                this.$__fetcher__ = this.$fetch();
            }
            return this.$__fetcher__.then((remoteSource) => remoteSource[fieldName || propertyKey]);
        };
        Object.defineProperty(prototype, propertyKey, { get });
    };
}
exports.lazy = lazy;
//# sourceMappingURL=ResolverContext.js.map